#!/bin/sh
curpath=`pwd`
scriptpath=${0%/*}
arguments=$*
tmpfile=tmp
shellopt=-x
ADBPATH=bin/adb
TARGETPKG=com.x0.strai.secondfrep
TARGETCLS=com.x0.strai.secondfrep.StrServ

if [ $0 = $scriptpath ] ; then
  scriptpath=.
fi

funcerros() {
  echo "FRep2 is available only for Android 7~."
  echo "For Android ~6, please consider previous edition."
  echo "[FRep] http://strai.x0.com/frep/"
  cd $curpath
  exit 1
}

funcerrnotgt() {
  echo "Cannot access the setup target, or FRep2 is not installed."
  echo "(For support) Please send the result of following command to support."
  echo "$scriptpath/$ADBPATH shell \"pm list packages -f | grep =$TARGETPKG\""
  cd $curpath
  exit 1
}

funcerr() {
  echo "FRep2 Precision mode setup failed. Please check followings."
  echo "- Connect your Device with Development | USB Debugging."
  echo "See also support page. http://strai.x0.com/frep2/"
  cd $curpath
  exit 1
}

funcoffline() {
  echo "Allow USB debugging on Android device. (Ctrl-C to cancel)"
  read tmpinput
}

funcnumeric() {
  numcand=$1
  expr "$numcand" + 1 >/dev/null 2>&1
  # -lt 2 Numeric
  if [ $? -ge 2 ] ; then
    numcand=0
  fi
  echo $numcand
# return $numcand
}

funcdevice() {
  echo "Multiple Android devices found."
  devnum=0
  $ADBPATH devices -l >$tmpfile
  # while~do~done with redirecting file because of variable lost.
  while read line ; do
    if [ -n "$line" ] ; then
      serial="`echo $line | awk '{print $1}'`"
#      serial=${line%%device*}
#      serial=${serial%% *  }
      product=${line#*product:}
      product=${product%transport_id*}
#      tid=${line#*transport_id}
      if [ $devnum -gt 0 ] ; then
        echo " ($devnum) $serial:$product"
#        echo " ($devnum) $serial:$product tid$tid"
      fi
      devnum=`expr $devnum + 1`
    fi
  done <$tmpfile

  echo "Enter the number of device to setup FRep2. (0 to cancel)"
  read selnum
  seldevice=
  devnum=0
  selnum=`funcnumeric $selnum`

  if [ $selnum -eq 0 ] ; then
    echo "Setup cancelled."
    exit 0
  else
    # while~do~done with redirecting file because of variable lost.
    while read line ; do
    if [ -n "$line" ] ; then
      if [ $devnum -eq $selnum ] ; then
        seldevice="`echo $line | awk '{print $1}'`"
        break
      fi
      devnum=`expr $devnum + 1`
    fi
    done <$tmpfile
  fi
  rm $tmpfile

  if [ -n $seldevice ] ; then
    arguments="-s $seldevice"
    echo "($selnum) $seldevice is selected."
  else
    echo "Retry with only single Android device connection, or designate"
    echo "single Android device by option -s (single serial from above)"
    echo "ex.) frep2.sh -s 00000000"
    exit 1
  fi
}

# get pid in ps -ef line.
getsecond() {
  case $# in
    0)
    return 0
    ;;
    1)
    return 0
    ;;
  esac
  echo $2

}
# Check process and kill if already running. (Android 8~, since ~7.1 does not have ps -ef)
killserv() {
  $ADBPATH $arguments shell "ps -ef | grep $TARGETCLS" >$tmpfile
  # while~do~done with redirecting file because of variable lost.
  while read line ; do
    case $line in
      *app_process*$TARGETCLS)
        PID=`getsecond $line`
        # echo $PID in $line
        case $PID in
          "");;
          0);;
          *)
          $ADBPATH $arguments shell kill $PID
          echo PID $PID strserv killed.
          ;;
        esac
        ;;
      *);;
    esac
  done <$tmpfile
  rm $tmpfile
}

cd $scriptpath
chmod +x $ADBPATH
$ADBPATH devices > /dev/null 2>&1

echo "[FRep2 Precision mode setup]"

# check Android version and USB debug connection
passed=0
while [ $passed -eq 0 ] ; do
  versionsdk=`$ADBPATH $arguments shell getprop ro.build.version.sdk 2>&1`
  case $? in
    # passed (no error)
    0)
    passed=1
    ;;
    # error
    *)
    case $versionsdk in
      # skip sdk check when getprop denied or not found.
      *enied)
      passed=1
      ;;
      *getprop*found)
      passed=1
      ;;
      # error: more than one device/emulator (continue)
      *emulator)
      funcdevice
      ;;
      # error: device offline (continue)
      *offline)
      funcoffline
      ;;
      # error: device unauthorized (continue)
      *unauthorized*)
      funcoffline
      ;;
      # other error
      *)
      echo $versionsdk
      funcerr
      ;;
    esac
    ;;
  esac
done

# not work on Android ~6
if [ $versionsdk -le 23 ] ; then
  funcerros
fi

# adjust option for Android 10 or later
if [ $versionsdk -ge 29 ] ; then
  shellopt=
fi

# force stop running process on Android 8~
if [ $versionsdk -ge 26 ] ; then
  killserv
fi

PKGPATH=`$ADBPATH $arguments shell pm path $TARGETPKG`
PKGPATHLEN=${#PKGPATH}
PKGPATH=`echo $PKGPATH | cut -c 9-$PKGPATHLEN`
ADBCMD="export CLASSPATH=$PKGPATH && app_process /system/bin $TARGETCLS"

if [ -n "$ADBCMD" ]; then
  echo "After StrServ Started, push Ctrl-C to exit setup script."
  echo ""
#  echo $ADBPATH $arguments shell $shellopt "$ADBCMD"
  echo "$ADBCMD &" | $ADBPATH $arguments shell $shellopt
else
  funcerrnotgt
fi

cd $curpath
exit 0

